setwd("各々の環境作業ディレクトリを指定してください")

# 必要なパッケージのインストール（初回のみ実行）
# Robyn: マーケティングミックスモデリング用
# reticulate: Pythonとの連携用
# install.packages("Robyn") #動作確認時バージョン1.40.0
# install.packages("reticulate") #動作確認時バージョン3.11.1

# ライブラリの読み込み
library(reticulate)
library(Robyn)

# バージョン確認
packageVersion("reticulate")
packageVersion("Robyn")

# Python環境の設定 (virtualenvを使用)
virtualenv_create("r-reticulate") #動作確認時Pythonバージョン3.9.6
use_virtualenv("r-reticulate", required = TRUE)

# Pythonのパス設定
use_python("~/.virtualenvs/r-reticulate/bin/python")

# Python用パッケージのインストール
py_install("nevergrad", pip = TRUE) #動作確認時バージョン1.0.8

# reticulateの設定確認（オプション）
# PythonとRの接続が正しく行われているか確認する場合に使用
reticulate::py_config()

# ファイルの作成設定
# FALSEにするとローカルファイルを作成せず、メモリ内で処理
create_files <- TRUE

# 結果を格納するフォルダを指定
# 事前にカレントディレクトリ内に"results"フォルダを作成しておく必要があります。
# ./resultsフォルダを作成する（存在しない場合）
if (!dir.exists("./results")) {
  dir.create("./results")
}
robyn_directory <- "./results"

################################################################
#### Step 1: Load data

# サンプルの入力データをCSVファイルから読み込みます。
dt_simulated = read.csv(file='各々のファイル配置ディレクトリを指定してください/sample.csv')
# 変換後のデータの先頭部分を確認

# 日本語の変数名は、プロットする際に文字化けする可能性があるため、英字（半角アルファベット）に変換します。
dt_simulated = dplyr::select(
  dt_simulated, 
  datetime = 1, 
  Net.Spend = 2, 
  Tv.Spend = 3, 
  temperature = 4, 
  rain = 5, 
  revenue = 6, 
  Weekend.FLG = 7, 
  dplyr::everything()
)

# 変換後のデータの先頭部分を確認
head(dt_simulated)

################################################################
#### Step 2a: 初回ユーザー向けモデル仕様設定（4ステップ）

#### 2a-1: 入力変数の指定

# メディアとオーガニック変数の係数は正の値になるよう設定
# その他の変数は正負どちらの符号も許容し、カスタマイズ可能

# 入力データを設定
InputCollect <- robyn_inputs(
  dt_input = dt_simulated,               # 元データ
  dt_holidays = dt_prophet_holidays,     # 祝日データ
  date_var = "datetime",                 # 日付変数（例: "2020-01-01"）
  dep_var = "revenue",                   # 従属変数（売上高やCVなど）
  dep_var_type = "revenue",              # 従属変数のタイプ（売上高かCVフラグか）
  prophet_vars = c("trend", "season", "holiday"),  # 時系列要素 ("trend", "season", "weekday", "holiday")
  prophet_country = "JP",                # 国名（日本の祝日はデフォルトで設定されていないため注意）
  context_vars = c("temperature", "rain", "Weekend.FLG"),  # イベント情報
  paid_media_spends = c("Net.Spend", "Tv.Spend"),          # メディア支出
  paid_media_vars = c("Net.Spend", "Tv.Spend"),            # メディア変数（支出または露出指標を使用）
  # organic_vars = c("newsletter"),    # PRなどの非広告メディア
  factor_vars = c("Weekend.FLG"),       # 因子変数（イベントなど）
  window_start = "2019-04-01",          # モデル構築に使用するデータの開始日
  window_end = "2020-03-31",            # モデル構築に使用するデータの終了日
  adstock = "geometric"                 # Adstock効果の形状
)
print(InputCollect)


#### 2a-2: ハイパーパラメータの定義

# 使用するハイパーパラメータの表示
hyper_names(adstock = InputCollect$adstock, all_media = InputCollect$all_media)

# Adstockと飽和曲線の形状を確認（必要に応じてプロットを表示）
plot_adstock(plot = FALSE)
plot_saturation(plot = FALSE)

# ハイパーパラメータの可変領域を設定
hyperparameters <- list(
  Net.Spend_alphas = c(0.5, 3),
  Net.Spend_gammas = c(0.3, 1),
  Net.Spend_thetas = c(0, 0.3),
  Tv.Spend_alphas = c(0.5, 3),
  Tv.Spend_gammas = c(0.3, 1),
  Tv.Spend_thetas = c(0.1, 0.4),
  train_size = 0.7 # ts-cvで使用するトレーニングデータの割合
)

#### 2a-3: ハイパーパラメータをrobyn_inputsに入力

# ハイパーパラメータを含むInputCollectオブジェクトを更新
InputCollect <- robyn_inputs(InputCollect = InputCollect, hyperparameters = hyperparameters)
print(InputCollect)

################################################################
#### Step 3: 初期モデルの構築

# モデルの全試行とイテレーションを実行

OutputModels <- robyn_run(
  InputCollect = InputCollect,  # モデル仕様を全て入力
  cores = NULL,                 # 並列計算を行う場合、使用するCPUコア数を指定
  iterations = 2000,            # 推奨: 2000回（ダミーデータセットに対して）
  trials = 5,                   # 推奨: 5回（ダミーデータセットに対して）
  ts_validation = TRUE,         # NRMSEを基準にクロスバリデーションを実施
  add_penalty_factor = FALSE     # 実験的？
)
print(OutputModels)

# 収束状況をplot
OutputModels$convergence$moo_distrb_plot
OutputModels$convergence$moo_cloud_plot

# クロスバリデーションplot
if (OutputModels$ts_validation) OutputModels$ts_validation_plot

## パレート最適解の計算, 結果とplotを指定フォルダにexport
OutputCollect <- robyn_outputs(
  InputCollect, OutputModels,
  pareto_fronts = 1, # automatically pick how many pareto-fronts to fill min_candidates (100)
  # min_candidates = 100, # top pareto models for clustering. Default to 100
  # calibration_constraint = 0.1, # range c(0.01, 0.1) & default at 0.1
  csv_out = "pareto", # "pareto", "all", or NULL (for none)
  clusters = TRUE, # Set to TRUE to cluster similar models by ROAS. See ?robyn_clusters
  export = create_files, # this will create files locally
  plot_folder = robyn_directory, # 指定したフォルダ内に結果ができる。当該フォルダがない場合cwdにできる
  plot_pareto = create_files # Set to FALSE to deactivate plotting and saving model one-pagers
)
print(OutputCollect)

# 必要なパッケージをインストールして読み込み
# install.packages("jsonlite") #動作確認時バージョン1.8.9
library(jsonlite)
packageVersion("jsonlite")

################################################################
#### Step 4: モデルの選択と保存

# 選択するモデルのIDを指定（上で実行したモデルから選択）
select_model <- "3_130_7"

# `OutputCollect` リストをJSON形式で保存
write_json(OutputCollect$allPareto$plotDataCollect[[select_model]], "OutputCollect.json")


# バージョン3.7.1以降: JSON形式でモデルをエクスポート（RDSファイルより高速で軽量）
ExportedModel <- robyn_write(
  InputCollect = InputCollect, 
  OutputCollect = OutputCollect, 
  select_model = select_model, 
  export = create_files
)
print(ExportedModel)

# 選択したモデルのOne-Pagerを作成
myOnePager <- robyn_onepagers(
  InputCollect = InputCollect, 
  OutputCollect = OutputCollect, 
  select_model = select_model, 
  export = FALSE
)

################################################################
#### Step 5: 選択したモデルで予算最適化を実施

# 選択したモデルのメディアサマリーを確認
print(ExportedModel)

# 注意: 制約条件の順序は次の順番で指定する必要があります
InputCollect$paid_media_spends

# シナリオ "max_response": 与えられた予算での売上最大化を目指した予算最適化を実施
# 例 1: date_rangeを指定しない場合、直近1カ月の予算最適化を行う
AllocatorCollect1 <- robyn_allocator(
  InputCollect = InputCollect,
  OutputCollect = OutputCollect,
  select_model = select_model,
  # date_range = NULL, # 指定しない場合は直近1カ月が対象期間となる
  total_budget = NULL, # 指定しない場合はdate_range内の総支出額がデフォルト
  channel_constr_low = c(0.7, 0.7, 0.7, 0.7), # メディア数と同じ長さで指定
  channel_constr_up = c(1.2, 1.2, 1.2, 1.2),  # メディア数と同じ長さで指定
  # channel_constr_multiplier = 3,  # メディア予算制約の拡張係数（必要なら指定）
  scenario = "max_response",        # シナリオ: 最大レスポンスを目指す
  export = create_files             # 結果をファイルにエクスポートするかどうか
)

# 最適化の結果を表示＆プロット
print(AllocatorCollect1)
plot(AllocatorCollect1)

# test
AllocatorCollect1 <- robyn_allocator(
  InputCollect = InputCollect,
  OutputCollect = OutputCollect,
  select_model = select_model,
  # date_range = c("2023-01-01", "2023-12-31"),  # 範囲として開始日と終了日を指定
  total_budget = NULL,
  channel_constr_low = c(0.7, 0.7, 0.7, 0.7),
  channel_constr_up = c(1.2, 1.2, 1.2, 1.2),
  scenario = "max_response",
  export = create_files
)

